@@ -1,9 +1,24 @@
 import _ from 'lodash';
 
 import { QueryCtrl } from 'app/plugins/sdk';
-import moment from 'moment';
-import { defaultQuery } from './StreamHandler';
+import { defaultQuery } from './runStreams';
 import { getBackendSrv } from 'app/core/services/backend_srv';
+import { dateTime } from '@grafana/data';
+
+export const defaultPulse: any = {
+  timeStep: 60,
+  onCount: 3,
+  onValue: 2,
+  offCount: 3,
+  offValue: 1,
+};
+
+export const defaultCSVWave: any = {
+  timeStep: 60,
+  valuesCSV: '0,0,2,2,1,1',
+};
+
+const showLabelsFor = ['random_walk', 'predictable_pulse', 'predictable_csv_wave'];
 
 export class TestDataQueryCtrl extends QueryCtrl {
   static templateUrl = 'partials/query.editor.html';
@@ -14,26 +29,29 @@ export class TestDataQueryCtrl extends QueryCtrl {
   newPointTime: any;
   selectedPoint: any;
 
+  showLabels = false;
+
   /** @ngInject */
   constructor($scope: any, $injector: any) {
     super($scope, $injector);
 
     this.target.scenarioId = this.target.scenarioId || 'random_walk';
     this.scenarioList = [];
-    this.newPointTime = moment();
+    this.newPointTime = dateTime();
     this.selectedPoint = { text: 'Select point', value: null };
+    this.showLabels = showLabelsFor.includes(this.target.scenarioId);
   }
 
   getPoints() {
     return _.map(this.target.points, (point, index) => {
       return {
-        text: moment(point[1]).format('MMMM Do YYYY, H:mm:ss') + ' : ' + point[0],
+        text: dateTime(point[1]).format('MMMM Do YYYY, H:mm:ss') + ' : ' + point[0],
         value: index,
       };
     });
   }
 
-  pointSelected(option) {
+  pointSelected(option: any) {
     this.selectedPoint = option;
   }
 
@@ -53,7 +71,7 @@ export class TestDataQueryCtrl extends QueryCtrl {
   $onInit() {
     return getBackendSrv()
       .get('/api/tsdb/testdata/scenarios')
-      .then(res => {
+      .then((res: any) => {
         this.scenarioList = res;
         this.scenario = _.find(this.scenarioList, { id: this.target.scenarioId });
       });
@@ -62,6 +80,7 @@ export class TestDataQueryCtrl extends QueryCtrl {
   scenarioChanged() {
     this.scenario = _.find(this.scenarioList, { id: this.target.scenarioId });
     this.target.stringInput = this.scenario.stringInput;
+    this.showLabels = showLabelsFor.includes(this.target.scenarioId);
 
     if (this.target.scenarioId === 'manual_entry') {
       this.target.points = this.target.points || [];
@@ -75,6 +94,18 @@ export class TestDataQueryCtrl extends QueryCtrl {
       delete this.target.stream;
     }
 
+    if (this.target.scenarioId === 'predictable_pulse') {
+      this.target.pulseWave = _.defaults(this.target.pulseWave || {}, defaultPulse);
+    } else {
+      delete this.target.pulseWave;
+    }
+
+    if (this.target.scenarioId === 'predictable_csv_wave') {
+      this.target.csvWave = _.defaults(this.target.csvWave || {}, defaultCSVWave);
+    } else {
+      delete this.target.csvWave;
+    }
+
     this.refresh();
   }
 
