@@ -1,13 +1,13 @@
 import _ from 'lodash';
 import $ from 'jquery';
-import moment from 'moment';
 import * as d3 from 'd3';
-import kbn from 'app/core/utils/kbn';
 import { appEvents, contextSrv } from 'app/core/core';
 import * as ticksUtils from 'app/core/utils/ticks';
 import { HeatmapTooltip } from './heatmap_tooltip';
 import { mergeZeroBuckets } from './heatmap_data_converter';
 import { getColorScale, getOpacityScale } from './color_scale';
+import { GrafanaThemeType, getColorFromHexRgbOrName, getValueFormat } from '@grafana/ui';
+import { toUtc } from '@grafana/data';
 
 const MIN_CARD_SIZE = 1,
   CARD_PADDING = 1,
@@ -19,7 +19,7 @@ const MIN_CARD_SIZE = 1,
   Y_AXIS_TICK_PADDING = 5,
   MIN_SELECTION_WIDTH = 2;
 
-export default function rendering(scope, elem, attrs, ctrl) {
+export default function rendering(scope: any, elem: any, attrs: any, ctrl: any) {
   return new HeatmapRenderer(scope, elem, attrs, ctrl);
 }
 export class HeatmapRenderer {
@@ -51,7 +51,7 @@ export class HeatmapRenderer {
   padding: any;
   margin: any;
   dataRangeWidingFactor: number;
-  constructor(private scope, private elem, attrs, private ctrl) {
+  constructor(private scope: any, private elem: any, attrs: any, private ctrl: any) {
     // $heatmap is JQuery object, but heatmap is D3
     this.$heatmap = this.elem.find('.heatmap-panel');
     this.tooltip = new HeatmapTooltip(this.$heatmap, this.scope);
@@ -89,7 +89,7 @@ export class HeatmapRenderer {
     this.clearCrosshair();
   }
 
-  onGraphHover(event) {
+  onGraphHover(event: { pos: any }) {
     this.drawSharedCrosshair(event.pos);
   }
 
@@ -116,7 +116,7 @@ export class HeatmapRenderer {
     }
   }
 
-  getYAxisWidth(elem) {
+  getYAxisWidth(elem: any) {
     const axisText = elem.selectAll('.axis-y text').nodes();
     const maxTextWidth = _.max(
       _.map(axisText, text => {
@@ -128,7 +128,7 @@ export class HeatmapRenderer {
     return maxTextWidth;
   }
 
-  getXAxisHeight(elem) {
+  getXAxisHeight(elem: any) {
     const axisLine = elem.select('.axis-x line');
     if (!axisLine.empty()) {
       const axisLinePosition = parseFloat(elem.select('.axis-x line').attr('y2'));
@@ -244,14 +244,13 @@ export class HeatmapRenderer {
   }
 
   // Wide Y values range and anjust to bucket size
-  wideYAxisRange(min, max, tickInterval) {
+  wideYAxisRange(min: number, max: number, tickInterval: number) {
     const yWiding = (max * (this.dataRangeWidingFactor - 1) - min * (this.dataRangeWidingFactor - 1)) / 2;
     let yMin, yMax;
 
     if (tickInterval === 0) {
       yMax = max * this.dataRangeWidingFactor;
       yMin = min - min * (this.dataRangeWidingFactor - 1);
-      tickInterval = (yMax - yMin) / 2;
     } else {
       yMax = Math.ceil((max + yWiding) / tickInterval) * tickInterval;
       yMin = Math.floor((min - yWiding) / tickInterval) * tickInterval;
@@ -350,7 +349,7 @@ export class HeatmapRenderer {
     this.ctrl.decimals = decimals;
 
     const tickValueFormatter = this.tickValueFormatter.bind(this);
-    function tickFormatter(valIndex) {
+    function tickFormatter(valIndex: string) {
       let valueFormatted = tsBuckets[valIndex];
       if (!_.isNaN(_.toNumber(valueFormatted)) && valueFormatted !== '') {
         // Try to format numeric tick labels
@@ -380,6 +379,12 @@ export class HeatmapRenderer {
     const posX = this.getYAxisWidth(this.heatmap) + Y_AXIS_TICK_PADDING;
     this.heatmap.select('.axis-y').attr('transform', 'translate(' + posX + ',' + posY + ')');
 
+    if (this.panel.yBucketBound === 'middle' && tickValues && tickValues.length) {
+      // Shift Y axis labels to the middle of bucket
+      const tickShift = 0 - this.chartHeight / (tickValues.length - 1) / 2;
+      this.heatmap.selectAll('.axis-y text').attr('transform', 'translate(' + 0 + ',' + tickShift + ')');
+    }
+
     // Remove vertical line in the right of axis labels (called domain in d3)
     this.heatmap
       .select('.axis-y')
@@ -388,10 +393,8 @@ export class HeatmapRenderer {
   }
 
   // Adjust data range to log base
-  adjustLogRange(min, max, logBase) {
-    let yMin, yMax;
-
-    yMin = this.data.heatmapStats.minLog;
+  adjustLogRange(min: number, max: number, logBase: number) {
+    let yMin = this.data.heatmapStats.minLog;
     if (this.data.heatmapStats.minLog > 1 || !this.data.heatmapStats.minLog) {
       yMin = 1;
     } else {
@@ -399,20 +402,20 @@ export class HeatmapRenderer {
     }
 
     // Adjust max Y value to log base
-    yMax = this.adjustLogMax(this.data.heatmapStats.max, logBase);
+    const yMax = this.adjustLogMax(this.data.heatmapStats.max, logBase);
 
     return { yMin, yMax };
   }
 
-  adjustLogMax(max, base) {
+  adjustLogMax(max: number, base: number) {
     return Math.pow(base, Math.ceil(ticksUtils.logp(max, base)));
   }
 
-  adjustLogMin(min, base) {
+  adjustLogMin(min: number, base: number) {
     return Math.pow(base, Math.floor(ticksUtils.logp(min, base)));
   }
 
-  logScaleTickValues(domain, base) {
+  logScaleTickValues(domain: any[], base: number) {
     const domainMin = domain[0];
     const domainMax = domain[1];
     const tickValues = [];
@@ -434,11 +437,11 @@ export class HeatmapRenderer {
     return tickValues;
   }
 
-  tickValueFormatter(decimals, scaledDecimals = null) {
+  tickValueFormatter(decimals: number, scaledDecimals: any = null) {
     const format = this.panel.yAxis.format;
-    return value => {
+    return (value: any) => {
       try {
-        return format !== 'none' ? kbn.valueFormats[format](value, decimals, scaledDecimals) : value;
+        return format !== 'none' ? getValueFormat(format)(value, decimals, scaledDecimals) : value;
       } catch (err) {
         console.error(err.message || err);
         return value;
@@ -517,19 +520,20 @@ export class HeatmapRenderer {
       const logBase = this.panel.yAxis.logBase;
       const domain = this.yScale.domain();
       const tickValues = this.logScaleTickValues(domain, logBase);
-      this.data.buckets = mergeZeroBuckets(this.data.buckets, _.min(tickValues));
+      this.data.buckets = mergeZeroBuckets(this.data.buckets, _.min(tickValues)!);
     }
 
     const cardsData = this.data.cards;
-    const maxValueAuto = this.data.cardStats.max;
-    const maxValue = this.panel.color.max || maxValueAuto;
-    const minValue = this.panel.color.min || 0;
-
-    const colorScheme = _.find(this.ctrl.colorSchemes, {
+    const cardStats = this.data.cardStats;
+    const maxValueAuto = cardStats.max;
+    const minValueAuto = Math.min(cardStats.min, 0);
+    const maxValue = _.isNil(this.panel.color.max) ? maxValueAuto : this.panel.color.max;
+    const minValue = _.isNil(this.panel.color.min) ? minValueAuto : this.panel.color.min;
+    const colorScheme: any = _.find(this.ctrl.colorSchemes, {
       value: this.panel.color.colorScheme,
     });
     this.colorScale = getColorScale(colorScheme, contextSrv.user.lightTheme, maxValue, minValue);
-    this.opacityScale = getOpacityScale(this.panel.color, maxValue);
+    this.opacityScale = getOpacityScale(this.panel.color, maxValue, minValue);
     this.setCardSize();
 
     let cards = this.heatmap.selectAll('.heatmap-card').data(cardsData);
@@ -551,17 +555,17 @@ export class HeatmapRenderer {
 
     const $cards = this.$heatmap.find('.heatmap-card');
     $cards
-      .on('mouseenter', event => {
+      .on('mouseenter', (event: any) => {
         this.tooltip.mouseOverBucket = true;
         this.highlightCard(event);
       })
-      .on('mouseleave', event => {
+      .on('mouseleave', (event: any) => {
         this.tooltip.mouseOverBucket = false;
         this.resetCardHighLight(event);
       });
   }
 
-  highlightCard(event) {
+  highlightCard(event: any) {
     const color = d3.select(event.target).style('fill');
     const highlightColor = d3.color(color).darker(2);
     const strokeColor = d3.color(color).brighter(4);
@@ -573,9 +577,8 @@ export class HeatmapRenderer {
       .style('stroke-width', 1);
   }
 
-  resetCardHighLight(event) {
-    d3
-      .select(event.target)
+  resetCardHighLight(event: any) {
+    d3.select(event.target)
       .style('fill', this.tooltip.originalFillColor)
       .style('stroke', this.tooltip.originalFillColor)
       .style('stroke-width', 0);
@@ -591,11 +594,12 @@ export class HeatmapRenderer {
       yGridSize = Math.floor((this.yScale(1) - this.yScale(base)) / splitFactor);
     }
 
-    this.cardWidth = xGridSize - this.cardPadding * 2;
+    const cardWidth = xGridSize - this.cardPadding * 2;
+    this.cardWidth = Math.max(cardWidth, MIN_CARD_SIZE);
     this.cardHeight = yGridSize ? yGridSize - this.cardPadding * 2 : 0;
   }
 
-  getCardX(d) {
+  getCardX(d: { x: any }) {
     let x;
     if (this.xScale(d.x) < 0) {
       // Cut card left to prevent overlay
@@ -607,25 +611,22 @@ export class HeatmapRenderer {
     return x;
   }
 
-  getCardWidth(d) {
-    let w;
+  getCardWidth(d: { x: any }) {
+    let w = this.cardWidth;
     if (this.xScale(d.x) < 0) {
       // Cut card left to prevent overlay
-      const cuttedWidth = this.xScale(d.x) + this.cardWidth;
-      w = cuttedWidth > 0 ? cuttedWidth : 0;
+      w = this.xScale(d.x) + this.cardWidth;
     } else if (this.xScale(d.x) + this.cardWidth > this.chartWidth) {
       // Cut card right to prevent overlay
       w = this.chartWidth - this.xScale(d.x) - this.cardPadding;
-    } else {
-      w = this.cardWidth;
     }
 
-    // Card width should be MIN_CARD_SIZE at least
-    w = Math.max(w, MIN_CARD_SIZE);
+    // Card width should be MIN_CARD_SIZE at least, but cut cards shouldn't be displayed
+    w = w > 0 ? Math.max(w, MIN_CARD_SIZE) : 0;
     return w;
   }
 
-  getCardY(d) {
+  getCardY(d: { y: number }) {
     let y = this.yScale(d.y) + this.chartTop - this.cardHeight - this.cardPadding;
     if (this.panel.yAxis.logBase !== 1 && d.y === 0) {
       y = this.chartBottom - this.cardHeight - this.cardPadding;
@@ -638,7 +639,7 @@ export class HeatmapRenderer {
     return y;
   }
 
-  getCardHeight(d) {
+  getCardHeight(d: { y: number }) {
     const y = this.yScale(d.y) + this.chartTop - this.cardHeight - this.cardPadding;
     let h = this.cardHeight;
 
@@ -663,15 +664,18 @@ export class HeatmapRenderer {
     return h;
   }
 
-  getCardColor(d) {
+  getCardColor(d: { count: any }) {
     if (this.panel.color.mode === 'opacity') {
-      return this.panel.color.cardColor;
+      return getColorFromHexRgbOrName(
+        this.panel.color.cardColor,
+        contextSrv.user.lightTheme ? GrafanaThemeType.Light : GrafanaThemeType.Dark
+      );
     } else {
       return this.colorScale(d.count);
     }
   }
 
-  getCardOpacity(d) {
+  getCardOpacity(d: { count: any }) {
     if (this.panel.color.mode === 'opacity') {
       return this.opacityScale(d.count);
     } else {
@@ -679,14 +683,14 @@ export class HeatmapRenderer {
     }
   }
 
-  getEventOffset(event) {
+  getEventOffset(event: any) {
     const elemOffset = this.$heatmap.offset();
     const x = Math.floor(event.clientX - elemOffset.left);
     const y = Math.floor(event.clientY - elemOffset.top);
     return { x, y };
   }
 
-  onMouseDown(event) {
+  onMouseDown(event: any) {
     const offset = this.getEventOffset(event);
     this.selection.active = true;
     this.selection.x1 = offset.x;
@@ -709,8 +713,8 @@ export class HeatmapRenderer {
       const timeTo = this.xScale.invert(Math.max(this.selection.x1, this.selection.x2) - this.yAxisWidth);
 
       this.ctrl.timeSrv.setTime({
-        from: moment.utc(timeFrom),
-        to: moment.utc(timeTo),
+        from: toUtc(timeFrom),
+        to: toUtc(timeTo),
       });
     }
 
@@ -722,7 +726,7 @@ export class HeatmapRenderer {
     this.clearCrosshair();
   }
 
-  onMouseMove(event) {
+  onMouseMove(event: any) {
     if (!this.heatmap) {
       return;
     }
@@ -743,10 +747,10 @@ export class HeatmapRenderer {
     }
   }
 
-  getEventPos(event, offset) {
+  getEventPos(event: { pageX: any; pageY: any }, offset: { x: any; y: any }) {
     const x = this.xScale.invert(offset.x - this.yAxisWidth).valueOf();
     const y = this.yScale.invert(offset.y - this.chartTop);
-    const pos = {
+    const pos: any = {
       pageX: event.pageX,
       pageY: event.pageY,
       x: x,
@@ -760,20 +764,20 @@ export class HeatmapRenderer {
     return pos;
   }
 
-  emitGraphHoverEvent(pos) {
+  emitGraphHoverEvent(pos: { panelRelY: number; offset: { y: number } }) {
     // Set minimum offset to prevent showing legend from another panel
     pos.panelRelY = Math.max(pos.offset.y / this.height, 0.001);
     // broadcast to other graph panels that we are hovering
     appEvents.emit('graph-hover', { pos: pos, panel: this.panel });
   }
 
-  limitSelection(x2) {
+  limitSelection(x2: number) {
     x2 = Math.max(x2, this.yAxisWidth);
     x2 = Math.min(x2, this.chartWidth + this.yAxisWidth);
     return x2;
   }
 
-  drawSelection(posX1, posX2) {
+  drawSelection(posX1: number, posX2: number) {
     if (this.heatmap) {
       this.heatmap.selectAll('.heatmap-selection').remove();
       const selectionX = Math.min(posX1, posX2);
@@ -800,7 +804,7 @@ export class HeatmapRenderer {
     }
   }
 
-  drawCrosshair(position) {
+  drawCrosshair(position: number) {
     if (this.heatmap) {
       this.heatmap.selectAll('.heatmap-crosshair').remove();
 
@@ -821,7 +825,7 @@ export class HeatmapRenderer {
     }
   }
 
-  drawSharedCrosshair(pos) {
+  drawSharedCrosshair(pos: { x: any }) {
     if (this.heatmap && this.ctrl.dashboard.graphTooltip !== 0) {
       const posX = this.xScale(pos.x) + this.yAxisWidth;
       this.drawCrosshair(posX);
